==> scala-k8s-client_2.12-0.5.1-sources.jar: CommonAPIs.scala <==
/*
 * Copyright 2022 Hossein Naderi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.hnaderi.k8s.client

import dev.hnaderi.k8s.utils._
import io.k8s.apimachinery.pkg.apis.meta.v1.APIGroupList
import io.k8s.apimachinery.pkg.apis.meta.v1.APIResourceList

import scala.concurrent.duration.FiniteDuration

abstract class ListingRequest[O: Decoder, COL: Decoder](
    url: String,
    allowWatchBookmarks: Option[Boolean] = None,
    continue: Option[String] = None,
    fieldSelector: List[String] = Nil,
    labelSelector: List[String] = Nil,
    limit: Option[Int] = None,
    resourceVersion: Option[String] = None,
    resourceVersionMatch: Option[String] = None,
    timeout: Option[FiniteDuration] = None
) extends HttpRequest[COL]
    with WatchRequest[WatchEvent[O]] {
  private def selector(name: String, l: List[String]) =
    if (l.isEmpty) None
    else Some(name -> l.mkString(", "))

  private def params: Seq[(String, String)] = Seq(
    continue.map(c => "continue" -> c),
    selector("fieldSelector", fieldSelector),
    selector("labelSelector", labelSelector),
    limit.map(l => "limit" -> l.toString),
    resourceVersion.map(v => "resourceVersion" -> v),
    resourceVersionMatch.map(v => "resourceVersionMatch" -> v),
    timeout.map(t => "timeoutSeconds" -> t.toSeconds.toString)
  ).flatten

  private def watchParams =
    Seq("watch" -> "true") ++
      allowWatchBookmarks.map(b => "allowWatchBookmarks" -> b.toString) ++
      params

  override def send[F[_]](
      http: HttpClient[F]
  ): F[COL] = http.get(url, params: _*)
  override def listen[F[_]](http: StreamingClient[F]) =
    http.connect(url, watchParams: _*)
}

abstract class GetRequest[O: Decoder](url: String) extends HttpRequest[O] {
  override def send[F[_]](http: HttpClient[F]): F[O] = http.get(url)
}

abstract class CreateRequest[RES: Encoder: Decoder](
    url: String,
    body: RES
) // TODO parameters
    extends HttpRequest[RES] {
  override def send[F[_]](
      http: HttpClient[F]
  ): F[RES] = http.post(url)(body)
}

abstract class PutRequest[IN: Encoder, OUT: Decoder](
    url: String,
    body: IN
) // TODO parameters
    extends HttpRequest[OUT] {
  override def send[F[_]](
      http: HttpClient[F]
  ): F[OUT] = http.put(url)(body)
}

abstract class DeleteRequest[OUT: Decoder](url: String) // TODO parameters
    extends HttpRequest[OUT] {
  override def send[F[_]](
      http: HttpClient[F]
  ): F[OUT] = http.delete(url)
}

abstract class APIResourceListingRequest(url: String)
    extends HttpRequest[APIResourceList] {
  override def send[F[_]](
      http: HttpClient[F]
  ): F[APIResourceList] = http.get(url)
}

abstract class APIGroupListingRequest(url: String)
    extends HttpRequest[APIGroupList] {
  override def send[F[_]](
      http: HttpClient[F]
  ): F[APIGroupList] = http.get(url)
}

==> scala-k8s-client_3-0.4.0-sources.jar: CommonAPIs.scala <==
/*
 * Copyright 2022 Hossein Naderi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.hnaderi.k8s.client

import dev.hnaderi.k8s.utils._
import io.k8s.apimachinery.pkg.apis.meta.v1.APIResourceList
import io.k8s.apimachinery.pkg.apis.meta.v1.APIGroupList

abstract class ListingRequest[O: Decoder, COL: Decoder](
    url: String
) // TODO parameters
    extends HttpRequest[COL]
    with WatchRequest[WatchEvent[O]] {
  override def send[F[_]](
      http: HttpClient[F]
  ): F[COL] = http.get(url)
  override def listen[F[_]](http: StreamingClient[F]) =
    http.connect(url, "watch" -> "true")
}

abstract class GetRequest[O: Decoder](url: String)
    extends ListingRequest[O, O](url)

abstract class CreateRequest[RES: Encoder: Decoder](
    url: String,
    body: RES
) // TODO parameters
    extends HttpRequest[RES] {
  override def send[F[_]](
      http: HttpClient[F]
  ): F[RES] = http.post(url)(body)
}

abstract class PutRequest[IN: Encoder, OUT: Decoder](
    url: String,
    body: IN
) // TODO parameters
    extends HttpRequest[OUT] {
  override def send[F[_]](
      http: HttpClient[F]
  ): F[OUT] = http.put(url)(body)
}

abstract class DeleteRequest[OUT: Decoder](url: String) // TODO parameters
    extends HttpRequest[OUT] {
  override def send[F[_]](
      http: HttpClient[F]
  ): F[OUT] = http.delete(url)
}

abstract class APIResourceListingRequest(url: String)
    extends HttpRequest[APIResourceList] {
  override def send[F[_]](
      http: HttpClient[F]
  ): F[APIResourceList] = http.get(url)
}

abstract class APIGroupListingRequest(url: String)
    extends HttpRequest[APIGroupList] {
  override def send[F[_]](
      http: HttpClient[F]
  ): F[APIGroupList] = http.get(url)
}
